/**
 * Post-install script to make the necessary modifications to Webtop config
 * files for the Precision web applications.
 */
function updateWebtopConfig()
{
	log( 2, "Precision MIB Browser install script updating Webtop configuration" );

	// Add the MIB Browser menu item to the Webtop alerts menu.
	try
	{
		addMIBBrowserMenuItem();
	}
	catch ( e )
	{
		log( 0, "Error making Precision MIB Browser additions to the Webtop alerts.nova file : " + e );
	}

	// Append the MIB Browser relevant, locale specific, menu properties to the Webtop versions.
	try
	{
		addMIBBrowserMenuProperties();
	}
	catch ( e )
	{
		log( 0, "Error making Precision MIB Browser menu property additions to the Webtop dynamicmenus.properties files : " + e );
	}

	log( 3, "Precision MIB Browser install script finished updating Webtop configuration" );
}


function revertWebtopConfig()
{
	log( 2, "Precision MIB Browser uninstall script updating Webtop configuration" );

	// Remove the MIB Browser menu item from the Webtop alerts menu.
	try
	{
		removeMIBBrowserMenuItem();
	}
	catch ( e )
	{
		log( 0, "Error removing Precision MIB Browser additions from the Webtop alerts.nova file : " + e );
	}

	// Remove the MIB Browser relevant, locale specific, menu properties from the Webtop versions.
	try
	{
		removeMIBBrowserMenuProperties();
	}
	catch ( e )
	{
		log( 0, "Error removing Precision MIB Browser menu property additions from the Webtop dynamicmenus.properties files : " + e );
	}

	log( 3, "Precision MIB Browser uninstall script finished updating Webtop configuration" );
}

/**
 * Makes an addition to the Webtop alerts.nova file to launch the MIB Browser
 * off from events in the AEL.
 */
function addMIBBrowserMenuItem()
{
	var alertsMenuFilename = getAlertsMenuFilename();

	log( 2, "Precision MIB Browser install checking Webtop alerts menu file " + alertsMenuFilename);

	// Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    // NOTE: This will not work with Mac!
    file.initWithPath( alertsMenuFilename );

    // Create an input stream to read in the file.
    var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();
	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
    	{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, adding in the Precision lines
			// if the file has not been modified already.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = false;
			do
			{
  				hasmore = inStream.readLine(line);
				if (line.value.indexOf("SNMPMIBBrowser") > -1)
				{
					alreadyModified = true;
				}
				else
				{
					if (line.value.indexOf('}') == -1)
					{
						newFileContents += line.value;
						newFileContents += getEndOfLineChars();
					}
					else
					{
						newFileContents += "tool(name=\"SNMPMIBBrowser\", label=\"Open SNMP MIB Browser\", mnemonic=\"\", shortcut=\"\", acl=\"*\")";
						newFileContents += getEndOfLineChars();
						newFileContents += "}";
					}
				}

			} while(hasmore);

			if (alreadyModified ==false)
			{
				// Backup existing alerts.nova to alerts.nova.orig
    			backupFile( alertsMenuFilename );

				// Write out the new alerts.nova file.
       			 writeToFile( alertsMenuFilename, newFileContents );
				log( 2, "alerts.nova modified by Precision MIB Browser install. Original version stored as alerts.nova.orig");
			}
    	}
    	catch ( e )
    	{
        	log( 0, e );
    	}
    	finally
    	{
        	inStream.close();
    	}
	}

}


/**
 * Removes the MIB Browser item from the Webtop alerts.nova file.
 */
function removeMIBBrowserMenuItem()
{
	var alertsMenuFilename = getAlertsMenuFilename();

	log( 2, "Precision MIB Browser uninstall checking Webtop alerts menu file " + alertsMenuFilename);

	var lineToBeDeleted = "SNMPMIBBrowser";

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	// NOTE: This will not work with Mac!
	file.initWithPath( alertsMenuFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line, checking to see if the file
			// contains the line to be deleted.
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = true;

			do
			{
				hasmore = inStream.readLine(line);
				if (line.value.indexOf(lineToBeDeleted) > -1)
				{
					alreadyModified = false;
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}


			} while(hasmore);

			if (alreadyModified == false)
			{
				// Backup existing alerts.nova to alerts.nova.orig
				backupFile( alertsMenuFilename );

				// Write out the alerts.nova file.
				writeToFile( alertsMenuFilename, newFileContents );

				log( 2, "SNMP MIB Browser menu item removed by Precision MIB Browser uninstall. Original version stored as alerts.nova.orig");
			}
		}
		catch ( e )
		{
			log( 0, e );
		}
		finally
		{
			inStream.close();
		}
	}
}


/**
 * Returns the filename, including the path, of the Webtop alerts menu file.
 */
function getAlertsMenuFilename()
{
	var alertsMenuFilename = getNchome();

	if ( ncipackage.shortArch == "win32" )
	{
		alertsMenuFilename += "\\etc\\webtop\\configstore\\ncwMenus\\alerts.nova";
	}
	else
	{
		alertsMenuFilename += "/etc/webtop/configstore/ncwMenus/alerts.nova";
	}

	return alertsMenuFilename;
}


/**
 * Makes any relevant additions to the Webtop dynamicmenus.properties files to localise the MIB Browser tool name.
 */
function addMIBBrowserMenuProperties()
{
	log( 2, "Precision MIB Browser install checking Webtop dynamicmenus.properties files." );

	// Create a list of dynamicmenus.properties files which are waiting to go into webtop.
	var newMenuFiles = getDynamicMenuFilenames();

	for( var i = 0; i < newMenuFiles.length; i++ )
	{
		modifyDynamicMenusProperties( newMenuFiles[i] );
	}
	
}

/**
 * Removes the lines for localising the MIB Browser tool name from dynamicmenus.properties.
 */
function removeMIBBrowserMenuProperties()
{
	log( 2, "Precision MIB Browser uninstall checking Webtop dynamicmenus.properties files." );
	
	// Create a list of dynamicmenus.properties files which have been modified by this install.
	var newMenuFiles = getDynamicMenuFilenames();

	for( var i = 0; i < newMenuFiles.length; i++ )
	{
		removeDynamicMenusProperties( newMenuFiles[i] );
	}
	
}

/**
 * Makes any relevant additions to the Webtop dynamicmenus.properties file to localise the MIB Browser tool name.
 */
function modifyDynamicMenusProperties( filename )
{
	var propFilename = getWebtopLocaleDir() + filename;

	var openingComments = getOpeningComments();

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);
	
			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = false;

			do
			{
				hasmore = inStream.readLine(line);

				if (line.value.indexOf(openingComments) > -1 )
				{
					alreadyModified = true;
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
			
			

			} while(hasmore);

			if (alreadyModified == false)
			{
				// read in from precision file and copy over new lines.
				newFileContents += getEndOfLineChars();
				newFileContents += readDynamicMenusProperties( filename );

				// Backup existing dynamicmenus.properties file to dynamicmenus.properties.orig
				backupFile( propFilename );

				// Write out the new dynamicmenus.properties file.
				writeToFile( propFilename, newFileContents );

				log( 2, "Dynamicmenus.properties has been updated with new properties. Original version stored as dynamicmenus.properties.orig");
			}
		}
		catch ( e )
		{
			log( 2, "DynamicMenus.properties file doesn't exist for this locale: " + propFilename );
		}
		finally
		{
			inStream.close();
		}
	}
}

/**
 * Reads in the new properties from the locales staging area.
 */
function readDynamicMenusProperties( filename )
{
	var propFilename = getDynamicMenusDir() + filename;

	var fileContents = "";

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);

			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line.
			var line = {}, hasmore;
		
			do
			{
				hasmore = inStream.readLine(line);

				fileContents += line.value;
				fileContents += getEndOfLineChars();
		
			} while(hasmore);

		}
		catch ( e )
		{
			log( 0, "Can not read in DynamicMenus.properties file from: " + propFilename );
			log( 0, e );
		}
		finally
		{
			inStream.close();
		}
	}

	return fileContents;
}

/**
 * Removes any relevant lines from the Webtop dynamicmenus.properties files which contain localises tool name properties.
 */
function removeDynamicMenusProperties( filename )
{
	var propFilename = getWebtopLocaleDir() + filename;

	var openingComments = getOpeningComments();
	var closingComments = getClosingComments();

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );

	// NOTE: This will not work with Mac!
	file.initWithPath( propFilename );

	// Create an input stream to read in the file.
	var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance();

	if (inStream)
	{
		inStream.QueryInterface(Components.interfaces.nsIFileInputStream);

		try
		{
			inStream.init(file, 0x01, 0444, 0);
	
			inStream.QueryInterface(Components.interfaces.nsILineInputStream);

			// read the file in line by line
			var line = {}, hasmore;
			var newFileContents = "";
			var alreadyModified = true;

			do
			{
				hasmore = inStream.readLine(line);

				if (line.value.indexOf(openingComments) > -1 )
				{
					alreadyModified = false;
	
					// remove this line and all other lines until "#END" string is found

					// Just do nothing, no need to add an empty line.
					//newFileContents += getEndOfLineChars();
								
					removeLines:
						do
						{
							hasmore = inStream.readLine(line);

							if (line.value.indexOf(closingComments) > -1 )
							{
								// Just do nothing, no need to add an empty line.
								//newFileContents += getEndOfLineChars();
								break removeLines;
							}
							else
							{
								// Just do nothing, no need to add an empty line.
								//newFileContents += getEndOfLineChars();
							}
				
						} while( hasmore )
				}
				else
				{
					newFileContents += line.value;
					newFileContents += getEndOfLineChars();
				}
			
			

			} while(hasmore);

			if (alreadyModified == false)
			{
				// Backup existing dynamicmenus.properties file to dynamicmenus.properties.orig
				backupFile( propFilename );

				// Write out the new dynamicmenus.properties file.
				writeToFile( propFilename, newFileContents );

				log( 2, "SNMP MIB Browser menu properties removed by Precision MIB Browser uninstall. Original version stored as dynamicmenus.properties.orig");
			}
		}
		catch ( e )
		{
			log( 2, "DynamicMenus.properties file doesn't exist for this locale: " + propFilename );
		}
		finally
		{
			inStream.close();
		}
	}
}

/**
 * Creates an array of filenames that are waiting to go into webtop. This list will need to be updated whenever a new locale file is created.
 */
function getDynamicMenuFilenames()
{
	// This could be written a bit more dynamically.
	if ( ncipackage.shortArch == "win32" )
	{
		var filenames = [ 
			"\\en\\dynamicmenus.properties",
			"\\en\\gb\\dynamicmenus.properties",
			"\\en\\us\\dynamicmenus.properties" ];
	}
	else
	{
		var filenames = [ 
			"/en/dynamicmenus.properties",
			"/en/gb/dynamicmenus.properties",
			"/en/us/dynamicmenus.properties" ];
	}

	return filenames;
}

/**
 * Gets the location of the locales staging area. This would be product specific.
 */
function getDynamicMenusDir()
{
	var dir = getNchome();

	if( ncipackage.shortArch == "win32" )
	{
		dir += "\\etc\\precision\\locale\\menus\\mibbrowser";
	}
	else
	{
		dir += "/etc/precision/locale/menus/mibbrowser";
	}

	return dir;
}

/**
 * Gets the location of the webtop locale files.
 */
function getWebtopLocaleDir()
{
	var dir = getNchome();

	if( ncipackage.shortArch == "win32" )
	{
		dir += "\\guifoundation\\webapps\\webtop\\WEB-INF\\classes\\locales";
	}
	else
	{
		dir += "/guifoundation/webapps/webtop/WEB-INF/classes/locales";
	}

	return dir;
}

/**
 * Gets the opening comments used in our locale files.
 */
function getOpeningComments()
{
	return "#Dynamic Menu Labels for ncp_mibbrowser tools";
}

/**
 * Gets the closing comments used in our locale files.
 */
function getClosingComments()
{
	return "#END";
}
