//
// IBM Tivoli Network Manager IP Edition
//
// IBM Confidential
// OCO Source Materials
//
// 5724-S45
//
// (C) Copyright IBM Corp. 1997, 2009
//
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
//

/**
********************************************************************************
*
* BackupHPMibs()
*
* If the user has obtained HP MIBs needed by the ITNM HP agents prior to their
* removal from the product, take copies of them to prevent their removal by the
* fix pack.
*
********************************************************************************
*/
function BackupHPMibs()
{
    var mibList = [ "CPQNIC.mib", "CPQHOST.mib", "CPQSINFO-MIB.mib", "CPQSTDEQ-MIB.mib", "HP-ICF-OID.mib", "HP-ICF-TC.mib"
, "STATISTICS.mib" ];

    for (var count = 0; count < mibList.length; ++count)
    {
        BackupSingleMib(mibList[count]);
    }
}

/**
********************************************************************************
*
* BackupSingleMib()
*
* Given the name of a MIB file, if it exists in $PRECISION_HOME/mibs, take a
* copy of it with the .backup extension.
*
* @param mibFileName The name of the MIB to be backed up.
*
********************************************************************************
*/
function BackupSingleMib(mibFileName)
{
    var fullPath = GetFullPlatformPath("precision/mibs/" + mibFileName);
    var backupPath = fullPath + ".backup";

    try
    {
        var file = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(fullPath);

        if (file.exists())
        {
            var backupFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            backupFile.initWithPath(backupPath);

            var backupStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            backupStream.init(backupFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

            var contents = ReadFile(file);

            backupStream.write(contents, contents.length);
            backupStream.close();

            Log(3, "Backed up " + fullPath + " to " + backupPath + ".");
        }
        else
        {
            Log(3, "Not backing up " + fullPath + " because it doesn't exist.");
        }
    }
    catch(e)
    {
        Log(0, e);
        Log(0, "Error whilst backing up " + fullPath + " to " + backupPath + ".");
    }
}

/**
********************************************************************************
*
* RestoreHPMibs()
*
* Restore any MIBs that were previously backed up by the BackupHPMibs()
* function.
*
********************************************************************************
*/
function RestoreHPMibs()
{
    var mibList = [ "CPQNIC.mib", "CPQHOST.mib", "CPQSINFO-MIB.mib", "CPQSTDEQ-MIB.mib", "HP-ICF-OID.mib", "HP-ICF-TC.mib", "STATISTICS.mib" ];

    for (var count = 0; count < mibList.length; ++count)
    {
        RestoreSingleMib(mibList[count]);
    }
}

/**
********************************************************************************
*
* RestoreSingleMib()
*
* Given the name of a MIB file, if a backup version of it exists in
* $PRECISION_HOME/mibs, duplicate it without the .backup extension and then
* remove the backup.
*
* @param mibFileName The name of the MIB to be restored.
*
********************************************************************************
*/
function RestoreSingleMib(mibFileName)
{
    var fullPath = GetFullPlatformPath("precision/mibs/" + mibFileName);
    var backupPath = fullPath + ".backup";

    try
    {
        var backupFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
        backupFile.initWithPath(backupPath);

        if (backupFile.exists())
        {
            var file = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            file.initWithPath(fullPath);

            var fileStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            fileStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate

            var contents = ReadFile(backupFile);

            fileStream.write(contents, contents.length);
            fileStream.close();

            backupFile.remove(false);

            Log(3, "Restored " + backupPath + " to " + fullPath + ".");
        }
        else
        {
            Log(3, "Not restoring " + fullPath + " because " + backupPath + " doesn't exist.");
        }
    }
    catch(e)
    {
        Log(0, e);
        Log(0, "Error whilst restoring " + backupPath + " to " + fullPath + ".");
    }
}

